﻿module Program

open System

type Name = string
type Number = string
type Entry = Name * Number
type PhoneBook = list<Entry>

// üres telefonkönyv létrehozása
let emptyBook : PhoneBook = []

// adott névhez tartozó telefonszám megkeresése (ha nincs ilyen név, akkor az eredmény Option None típusú lesz)
let rec findEntry (p : PhoneBook) (n : Name) : (Number option) =
    match List.tryFind (fun ((name, number)) -> name = n) p with
    | None -> None
    | Some (name, number) -> Some number

// bejegyzés beszúrása rendezett módon a telefonkönyvbe (ha a bejegyzés már létezik, akkor azt frissíti)
let rec addEntry (p : PhoneBook) (e : Entry) : PhoneBook =
    match p with
    | [] -> [e]
    | (name, number) as entry :: book when name < (fst e) -> entry :: addEntry book e
    | (name, number) :: book when name = (fst e) -> e :: book
    | x :: book -> [e; x] @ book

// menü kiírása a konzolra
let printHelp () : unit =
    printfn "Commands:"
    printfn "? - search"
    printfn "+ - new entry"
    printfn "s - save phonebook"
    printfn "l - load phonebook"
    printfn "n - print names"
    printfn "q - quit"

// fogadójel kiírása, majd egy parancs bekérése
let prompt (s : string) : string =
    printf "%s" s
    System.Console.ReadLine()

// adott névhez tartozó telefonszám megkeresése és az eredmény kiírása
// (a keresés addig megy, amíg üres nevet nem adunk meg)
let rec findAction (p : PhoneBook) : unit =
    let n = prompt "name> "
    if n = "" then ()
    else
        match findEntry p n with
        | Some number -> printfn "number: %s" ((List.find (fun ((name, number)) -> name = n) p) |> snd)
        | None -> printfn "The name is not in the book."
        findAction p

// bejegyzések hozzáadása a telefonkönyvhöz addig, amíg üres nevet nem adunk meg
let rec addAction (p : PhoneBook) : PhoneBook =
    let name = prompt "name> "
    if name = "" then p
    else
        let number = prompt "number> "
        addAction (addEntry p (name, number))

// telefonkönyv elmentése fájlba
let saveAction (p : PhoneBook) : unit =
    System.IO.File.WriteAllLines("phonebook.book", Seq.map (fun ((x, y)) -> x + "," + y) (List.toSeq p))

// telefonkönyv betöltése fájlból
let loadAction () : PhoneBook =
    System.IO.File.ReadAllLines(prompt "path> ")
    |> List.ofArray
    |> List.map (fun x -> x.Split(','))
    |> List.map (fun x -> (x.[0], x.[1]))

// telefonkönyv neveinek kiírása konzolra
let printNames (p : PhoneBook) : unit =
    printfn "%A" (List.map fst p)

// parancsok feldolgozása
let rec eventLoop (p : PhoneBook) : unit =
    match prompt "command> " with
    | "?" ->
        findAction p
        eventLoop p
    | "+" ->
        addAction p |> eventLoop
    | "s" ->
        saveAction p
        printfn "The phonebook has saved."
        eventLoop p
    | "l" ->
        let p = loadAction ()
        printfn "Loaded %d entries." p.Length
        eventLoop p
    | "n" ->
        printNames p
        eventLoop p
    | "q" -> ()
    | _ ->
        printHelp ()
        eventLoop p
        

let main () : unit =
    printfn "--- Phone Book Register ---"
    printHelp ()
    eventLoop emptyBook

main ()